<?php
if (!defined("ROOT_PATH"))
{
	header("HTTP/1.1 403 Forbidden");
	exit;
}
class pjAppController extends pjController
{
	public $models = array();

	public $defaultLocale = 'admin_locale_id';
	
	public $defaultFields = 'fields';
	
	public $defaultFieldsIndex = 'fields_index';
	
	protected function loadSetFields($force=FALSE, $locale_id=NULL, $fields=NULL)
	{
		if (is_null($locale_id))
		{
			$locale_id = $this->getLocaleId();
		}
	
		if (is_null($fields))
		{
			$fields = $this->defaultFields;
		}
	
		$registry = pjRegistry::getInstance();
		if ($force
				|| !isset($_SESSION[$this->defaultFieldsIndex])
				|| $_SESSION[$this->defaultFieldsIndex] != $this->option_arr['o_fields_index']
				|| !isset($_SESSION[$fields])
				|| empty($_SESSION[$fields]))
		{
			pjAppController::setFields($locale_id);
	
			# Update session
			if ($registry->is('fields'))
			{
				$_SESSION[$fields] = $registry->get('fields');
			}
			$_SESSION[$this->defaultFieldsIndex] = $this->option_arr['o_fields_index'];
		}
	
		if (isset($_SESSION[$fields]) && !empty($_SESSION[$fields]))
		{
			# Load fields from session
			$registry->set('fields', $_SESSION[$fields]);
		}

		return TRUE;
	}
	
	public function isEditor()
    {
   		return $this->getRoleId() == 2;
    }
	
    public function isOneAdminReady()
    {
    	return $this->isAdmin();
    }
    public function isCountryReady()
    {
    	return $this->isAdmin();
    }
	public static function setTimezone($timezone="UTC")
    {
    	if (in_array(version_compare(phpversion(), '5.1.0'), array(0,1)))
		{
			date_default_timezone_set($timezone);
		} else {
			$safe_mode = ini_get('safe_mode');
			if ($safe_mode)
			{
				putenv("TZ=".$timezone);
			}
		}
    }

	public static function setMySQLServerTime($offset="-0:00")
    {
		pjAppModel::factory()->prepare("SET SESSION time_zone = :offset;")->exec(compact('offset'));
    }
    
	public function setTime()
	{
		if (isset($this->option_arr['o_timezone']))
		{
			$offset = $this->option_arr['o_timezone'] / 3600;
			if ($offset > 0)
			{
				$offset = "-".$offset;
			} elseif ($offset < 0) {
				$offset = "+".abs($offset);
			} elseif ($offset === 0) {
				$offset = "+0";
			}
	
			pjAppController::setTimezone('Etc/GMT' . $offset);
			if (strpos($offset, '-') !== false)
			{
				$offset = str_replace('-', '+', $offset);
			} elseif (strpos($offset, '+') !== false) {
				$offset = str_replace('+', '-', $offset);
			}
			pjAppController::setMySQLServerTime($offset . ":00");
		}
	}
    
    public function beforeFilter()
    {
    	$this->appendJs('jquery.min.js', PJ_THIRD_PARTY_PATH . 'jquery/');
		$dm = new pjDependencyManager(PJ_THIRD_PARTY_PATH);
		$dm->load(PJ_CONFIG_PATH . 'dependencies.php')->resolve();
		$this->appendJs('jquery-migrate.min.js', $dm->getPath('jquery_migrate'), FALSE, FALSE);
		$this->appendJs('pjAdminCore.js');
		$this->appendCss('reset.css');
		 
		$this->appendJs('js/jquery-ui.custom.min.js', PJ_THIRD_PARTY_PATH . 'jquery_ui/');
		$this->appendCss('css/smoothness/jquery-ui.min.css', PJ_THIRD_PARTY_PATH . 'jquery_ui/');
		
		$this->appendCss('pj-all.css', PJ_FRAMEWORK_LIBS_PATH . 'pj/css/');
		$this->appendCss('admin.css');
		
    	if ($_GET['controller'] != 'pjInstaller')
		{
			$this->models['Option'] = pjOptionModel::factory();
			$this->option_arr = $this->models['Option']->getPairs($this->getForeignId());
			$this->set('option_arr', $this->option_arr);
			$this->setTime();
			
			if (!isset($_SESSION[$this->defaultLocale])) {
				$locale_arr = pjLocaleModel::factory()->where('is_default', 1)->limit(1)->findAll()->getData();
				if (count($locale_arr) === 1){
					$this->setLocaleId($locale_arr[0]['id']);
				}
			}
			$this->loadSetFields();
		}
    }
    
    public function pjActionCheckInstall()
    {
    	$this->setLayout('pjActionEmpty');
    
    	$result = array('status' => 'OK', 'code' => 200, 'text' => 'Operation succeeded', 'info' => array());
    	$folders = array(	
    			'app/web/upload', 
    			'app/web/upload/bookings', 
    			'app/web/upload/events', 
    			'app/web/upload/events/thumb', 
    			'app/web/upload/tickets', 
    			'app/web/upload/tickets/barcodes', 
    			'app/web/upload/tickets/pdfs'
    	);
    	foreach ($folders as $dir)
    	{
    		if (!is_writable($dir))
    		{
    			$result['status'] = 'ERR';
    			$result['code'] = 101;
    			$result['text'] = 'Permission requirement';
    			$result['info'][] = sprintf('Folder \'<span class="bold">%1$s</span>\' is not writable. You need to set write permissions (chmod 777) to directory located at \'<span class="bold">%1$s</span>\'', $dir);
    		}
    	}
    
    	return $result;
    }
    
    public function getForeignId()
    {
    	return 1;
    }
    
    public static function setFields($locale)
    {
    	if(isset($_SESSION['lang_show_id']) && (int) $_SESSION['lang_show_id'] == 1)
		{
			$fields = pjMultiLangModel::factory()
				->select('CONCAT(t1.content, CONCAT(":", t2.id, ":")) AS content, t2.key')
				->join('pjField', "t2.id=t1.foreign_id", 'inner')
				->where('t1.locale', $locale)
				->where('t1.model', 'pjField')
				->where('t1.field', 'title')
				->findAll()
				->getDataPair('key', 'content');
		}else{
			$fields = pjMultiLangModel::factory()
				->select('t1.content, t2.key')
				->join('pjField', "t2.id=t1.foreign_id", 'inner')
				->where('t1.locale', $locale)
				->where('t1.model', 'pjField')
				->where('t1.field', 'title')
				->findAll()
				->getDataPair('key', 'content');
		}
		$registry = pjRegistry::getInstance();
		$tmp = array();
		if ($registry->is('fields'))
		{
			$tmp = $registry->get('fields');
		}
		$arrays = array();
		foreach ($fields as $key => $value)
		{
			if (strpos($key, '_ARRAY_') !== false)
			{
				list($prefix, $suffix) = explode("_ARRAY_", $key);
				if (!isset($arrays[$prefix]))
				{
					$arrays[$prefix] = array();
				}
				$arrays[$prefix][$suffix] = $value;
			}
		}
		require PJ_CONFIG_PATH . 'settings.inc.php';
		$fields = array_merge($tmp, $fields, $settings, $arrays);
		$registry->set('fields', $fields);
    }

    public static function jsonDecode($str)
	{
		$Services_JSON = new pjServices_JSON();
		return $Services_JSON->decode($str);
	}
	
	public static function jsonEncode($arr)
	{
		$Services_JSON = new pjServices_JSON();
		return $Services_JSON->encode($arr);
	}
	
	public static function jsonResponse($arr)
	{
		header("Content-Type: application/json; charset=utf-8");
		echo pjAppController::jsonEncode($arr);
		exit;
	}

	public function getLocaleId()
	{
		return isset($_SESSION[$this->defaultLocale]) && (int) $_SESSION[$this->defaultLocale] > 0 ? (int) $_SESSION[$this->defaultLocale] : false;
	}
	
	public function setLocaleId($locale_id)
	{
		$_SESSION[$this->defaultLocale] = (int) $locale_id;
	}
	
	public function friendlyURL($str, $divider='-')
	{
		$str = mb_strtolower($str, mb_detect_encoding($str)); // change everything to lowercase
		$str = trim($str); // trim leading and trailing spaces
		$str = preg_replace('/[_|\s]+/', $divider, $str); // change all spaces and underscores to a hyphen
		$str = preg_replace('/\x{00C5}/u', 'AA', $str);
		$str = preg_replace('/\x{00C6}/u', 'AE', $str);
		$str = preg_replace('/\x{00D8}/u', 'OE', $str);
		$str = preg_replace('/\x{00E5}/u', 'aa', $str);
		$str = preg_replace('/\x{00E6}/u', 'ae', $str);
		$str = preg_replace('/\x{00F8}/u', 'oe', $str);
		$str = preg_replace('/[^a-z\x{0400}-\x{04FF}0-9-]+/u', '', $str); // remove all non-cyrillic, non-numeric characters except the hyphen
		$str = preg_replace('/[-]+/', $divider, $str); // replace multiple instances of the hyphen with a single instance
		$str = preg_replace('/^-+|-+$/', '', $str); // trim leading and trailing hyphens
		return $str;
	}
	
	protected function setPrice($current_id, $id, $post)
	{
		$pjPriceModel = pjPriceModel::factory();
		$pjMultiLangModel = pjMultiLangModel::factory();
		
		if(isset($post['index_arr']) && $post['index_arr'] != '')
		{
			$index_arr = explode("|", $post['index_arr']);
			foreach($index_arr as $k => $v)
			{
				if(strpos($v, 'ebc') !== false)
				{
					$price_data = array();
					$price_data['event_id'] = $id;
					$price_data['recurring'] = md5($current_id. $v . PJ_SALT);
					$price_data['price'] = $post['price'][$v];
					$price_data['available'] = $post['available'][$v];
					
					$price_id = $pjPriceModel->reset()->setAttributes($price_data)->insert()->getInsertId();
					if ($price_id !== false && (int) $price_id > 0)
					{
						foreach ($post['i18n'] as $locale => $locale_arr)
						{
							foreach ($locale_arr as $field => $content)
							{
								if(is_array($content))
								{
									$insert_id = $pjMultiLangModel->reset()->setAttributes(array(
											'foreign_id' => $price_id,
											'model' => 'pjPrice',
											'locale' => $locale,
											'field' => $field,
											'content' => $content[$v],
											'source' => 'data'
									))->insert()->getInsertId();
								}
							}
						}
					}
				}
			}
		}
	}
	
	protected function updatePrice($event_id, $post, $has_recurring)
	{
		$pjPriceModel = pjPriceModel::factory();
		$pjEventModel = pjEventModel::factory();
		$pjMultiLangModel = pjMultiLangModel::factory();
		
		$index_arr = explode("|", $post['index_arr']);
		foreach($index_arr as $k => $v)
		{
			if(strpos($v, 'ebc') !== false)
			{
				if($has_recurring == 1)
				{
					$recurring_events_arr = $pjEventModel->reset()->where('recurring_id', $post['recurring_id'])->findAll()->getData();
					
					foreach ($recurring_events_arr as $event)
					{
						$price_data = array();
						$price_data['event_id'] = $event['id'];
						$price_data['recurring'] = md5($event_id. $v . PJ_SALT);
						$price_data['price'] = $post['price'][$v];
						$price_data['available'] = $post['available'][$v];
						$price_id = $pjPriceModel->reset()->setAttributes($price_data)->insert()->getInsertId();
						if ($price_id !== false && (int) $price_id > 0)
						{
							foreach ($post['i18n'] as $locale => $locale_arr)
							{
								foreach ($locale_arr as $field => $content)
								{
									if(is_array($content))
									{
										$insert_id = $pjMultiLangModel->reset()->setAttributes(array(
												'foreign_id' => $price_id,
												'model' => 'pjPrice',
												'locale' => $locale,
												'field' => $field,
												'content' => $content[$v],
												'source' => 'data'
										))->insert()->getInsertId();
									}
								}
							}
						}
					}
				}else{
					$price_data = array();
					$price_data['event_id'] = $event_id;
					$price_data['recurring'] = md5($event_id. $v . PJ_SALT);
					$price_data['price'] = $post['price'][$v];
					$price_data['available'] = $post['available'][$v];
					$price_id = $pjPriceModel->reset()->setAttributes($price_data)->insert()->getInsertId();
					if ($price_id !== false && (int) $price_id > 0)
					{
						foreach ($post['i18n'] as $locale => $locale_arr)
						{
							foreach ($locale_arr as $field => $content)
							{
								if(is_array($content))
								{
									$insert_id = $pjMultiLangModel->reset()->setAttributes(array(
											'foreign_id' => $price_id,
											'model' => 'pjPrice',
											'locale' => $locale,
											'field' => $field,
											'content' => $content[$v],
											'source' => 'data'
									))->insert()->getInsertId();
								}
							}
						}
					}
				}
			}else{
				if($has_recurring == 1)
				{
					$price = $pjPriceModel->reset()->find($v)->getData();
					$recurring_price_arr = $pjPriceModel->reset()->where('recurring', $price['recurring'])->findAll()->getData();
					foreach($recurring_price_arr as $rp)
					{
						$price_data = array();
						$price_data['price'] = $post['price'][$v];
						$price_data['available'] = $post['available'][$v];
						$pjPriceModel->reset()->where('id', $rp['id'])->limit(1)->modifyAll($price_data);
						
						foreach ($post['i18n'] as $locale => $locale_arr)
						{
							foreach ($locale_arr as $field => $content)
							{
								if(is_array($content))
								{
									$sql = sprintf("INSERT INTO `%1\$s` (`id`, `foreign_id`, `model`, `locale`, `field`, `content`, `source`)
													VALUES (NULL, :foreign_id, :model, :locale, :field, :update_content, :source)
													ON DUPLICATE KEY UPDATE `content` = :update_content, `source` = :source;",
											$pjMultiLangModel->getTable()
									);
									$foreign_id = $rp['id'];
									$model = 'pjPrice';
									$source = 'data';
									$update_content = $content[$v];
									$modelObj = $pjMultiLangModel->reset()->prepare($sql)->exec(compact('foreign_id', 'model', 'locale', 'field', 'update_content', 'source'));
									if ($modelObj->getAffectedRows() > 0 || $modelObj->getInsertId() > 0)
									{
						
									}
								}
							}
						}
					}
				}else{
					$price_data = array();
					$price_data['price'] = $post['price'][$v];
					$price_data['available'] = $post['available'][$v];
					$pjPriceModel->reset()->where('id', $v)->limit(1)->modifyAll($price_data);
					foreach ($post['i18n'] as $locale => $locale_arr)
					{
						foreach ($locale_arr as $field => $content)
						{
							if(is_array($content))
							{
								$sql = sprintf("INSERT INTO `%1\$s` (`id`, `foreign_id`, `model`, `locale`, `field`, `content`, `source`)
													VALUES (NULL, :foreign_id, :model, :locale, :field, :update_content, :source)
													ON DUPLICATE KEY UPDATE `content` = :update_content, `source` = :source;",
										$pjMultiLangModel->getTable()
								);
								$foreign_id = $v;
								$model = 'pjPrice';
								$source = 'data';
								$update_content = $content[$v];
								$modelObj = $pjMultiLangModel->reset()->prepare($sql)->exec(compact('foreign_id', 'model', 'locale', 'field', 'update_content', 'source'));
								if ($modelObj->getAffectedRows() > 0 || $modelObj->getInsertId() > 0)
								{
										
								}
							}
						}
					}
				}
				
			}
		}
		if(isset($post['remove_arr']) && $post['remove_arr'] != '')
		{
			$remove_arr = explode("|", $post['remove_arr']);
			if($has_recurring == 1)
			{
				$price_arr = $pjPriceModel->reset()->whereIn('id', $remove_arr)->findAll()->getData();
				foreach($price_arr as $price)
				{
					$recurring_price_arr = $pjPriceModel->reset()->where('recurring', $price['recurring'])->findAll()->getData();
					foreach($recurring_price_arr as $rp)
					{
						$pjMultiLangModel->reset()->where('model', 'pjPrice')->whereIn('foreign_id', $rp['id'])->eraseAll();
					}
					$pjPriceModel->reset()->where('recurring', $price['recurring'])->eraseAll();
				}
			}else{
				$pjMultiLangModel->reset()->where('model', 'pjPrice')->whereIn('foreign_id', $remove_arr)->eraseAll();
				$pjPriceModel->reset()->whereIn('id', $remove_arr)->eraseAll();
			}
		}
	}
	
	protected function getFromEmail()
	{
		$arr = pjUserModel::factory()->find(1)->getData();
		return $arr['email'];
	}
	
	protected function bookingEmail($booking_arr, $opt)
	{
		if (!in_array($opt, array('confirmation', 'payment')))
		{
			return false;
		}
		if(empty($booking_arr['o_email_'. $opt .'_subject']) || empty($booking_arr['o_email_' . $opt]))
		{
			return false;
		}
		$pjBookingDetailModel = pjBookingDetailModel::factory();
		
		$event_date = pjUtil::getEventDateTime($booking_arr['event_start_ts'], $booking_arr['event_end_ts'], $this->option_arr['o_date_format'], $this->option_arr['o_time_format'], $booking_arr['o_show_start_time'], $booking_arr['o_show_end_time']);
		
		$cancelURL = PJ_INSTALL_URL . 'index.php?controller=pjFront&action=pjActionCancel&id='.$booking_arr['id'].'&hash='.md5($booking_arr['id'].$booking_arr['created'].PJ_SALT);
		
		$pdf_tickets = PJ_INSTALL_URL . PJ_UPLOAD_PATH . 'tickets/pdfs/p_' . $booking_arr['unique_id'] . '.pdf';
		
		$event = $booking_arr['event_title']. ' | ' . $event_date;

		$booking_detail_arr = pjBookingDetailModel::factory()
			->select('t1.*, t2.content as price_name')
			->join('pjMultiLang', "t2.foreign_id = t1.price_id AND t2.model = 'pjPrice' AND t2.locale = '".$this->getLocaleId()."' AND t2.field = 'name'", 'left')
			->where('t1.booking_id', $booking_arr['id'])
			->findAll()->getData();

        $category_arr = pjCategoryModel::factory()-> find($booking_arr['category_id'])->getData();
		
		$tickets = "\n";
		foreach($booking_detail_arr as $v)
		{
			$tickets .= $v['price_name'] . ': ' . $v['cnt'] . " x " . $v['unit_price']  . ' ' . $this->option_arr['o_currency'] . "\n";
		}
		$total = $booking_arr['booking_total'] . ' ' . $this->option_arr['o_currency'];
		$tax = $booking_arr['booking_tax'] . ' ' . $this->option_arr['o_currency'];
		$deposit = $booking_arr['booking_deposit'] . ' ' . $this->option_arr['o_currency'];
		$balance = ($booking_arr['booking_total'] - $booking_arr['booking_deposit']) . ' ' . $this->option_arr['o_currency'];
		
		$search = array('{Name}', '{Email}', '{Phone}', '{Country}', '{City}', '{State}', '{Zip}', '{Address}', '{Tickets}', '{PDF_Tickets}', '{Notes}', '{CCType}', '{CCNum}', '{CCExp}', '{CCSec}', '{PaymentMethod}', '{Event}', '{EventTitle}', '{EventDateTime}', '{EventLocation}', '{Total}', '{Tax}', '{Deposit}', '{Balance}', '{BookingID}', '{CancelURL}');
		$replace = array($booking_arr['customer_name'], $booking_arr['customer_email'], $booking_arr['customer_phone'], $booking_arr['country_title'], $booking_arr['customer_city'], $booking_arr['customer_state'], $booking_arr['customer_zip'], $booking_arr['customer_address'], $tickets, $pdf_tickets, $booking_arr['customer_notes'], $booking_arr['cc_type'], $booking_arr['cc_num'], ($booking_arr['payment_method'] == 'creditcard' ? $booking_arr['cc_exp'] : NULL), $booking_arr['cc_code'], $booking_arr['payment_method'], $event, $booking_arr['event_title'], $event_date, $booking_arr['event_location'], $total, $tax, $deposit, $balance, $booking_arr['unique_id'], $cancelURL);
		
		$pjEmail = new pjEmail();
					
		if ($this->option_arr['o_send_email'] == 'smtp')
		{
			$pjEmail
				->setTransport('smtp')
				->setSmtpHost($this->option_arr['o_smtp_host'])
				->setSmtpPort($this->option_arr['o_smtp_port'])
				->setSmtpUser($this->option_arr['o_smtp_user'])
				->setSmtpPass($this->option_arr['o_smtp_pass'])
			;
		}
		$from = $this->getFromEmail();
		$pjEmail->setContentType('text/plain');
		$pjEmail->setFrom($from);
		
		$subject = str_replace($search, $replace, $booking_arr['o_email_'. $opt .'_subject']);
		$message = str_replace($search, $replace, $booking_arr['o_email_' . $opt]);
		
		$pjEmail->setSubject($subject);
		
		$admin_arr = pjUserModel::factory()->where('role_id', 1)->findAll()->getData();
		
		#send to all administrator
		foreach($admin_arr as $admin)
		{
			$to = $admin['email'];
			$pjEmail->setTo($to);
			$pjEmail->send($message);
		}

        $to = $category_arr['email'];
        $pjEmail->setTo($to);
        $pjEmail->send($message);

		
		$to = $booking_arr['customer_email'];
		$pjEmail->setTo($to);
		$pjEmail->send($message);
		
	}
	
	protected function getTicketInfo($params){
		$ticket_info = '';
		$ticket_price = $params['price_name'] . " " .  __('lblTicket', true) . ", " . pjUtil::formatCurrencySign($params['unit_price'], $this->option_arr['o_currency']) . "<br/>";
		if(empty($params['ticket_detail']))
		{
			$ticket_info .= $params['customer_name'] . "<br/>";
			$ticket_info .= $params['customer_email'] . "<br/>";
			$ticket_info .= $ticket_price;
		}else{
			$search = array('{Name}', '{Email}','{Ticket}', '{EventTitle}', '{EventDateTime}');
			$replace = array($params['customer_name'], $params['customer_email'], $ticket_price, $params['event_title'], $params['event_datetime']);
			$ticket_info = str_replace($search, $replace, $params['ticket_detail']);
			$ticket_info = preg_replace('/\r\n|\n/', '<br />', $ticket_info);
		}
		
		return $ticket_info;
	}
}
?>